﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using iqiyiWin.Api;
using iqiyiWin.Model;
using iqiyiWin.Util;
using iqiyiWin.Constant;
using System.Threading;
using System.Collections;
using System.IO;

namespace iqiyiWin
{
    public partial class Main : Form, IUILog
    {
        /// <summary>
        /// 是否调试
        /// </summary>
        public static Boolean isDebuger = false;
        /// <summary>
        /// 爱奇艺账号列表
        /// </summary>
        public List<IqiyiAccount> IqiyiAccounts = new List<IqiyiAccount>();
        /// <summary>
        /// 选中数量
        /// </summary>
        public static int SelectNum = 0;
        /// <summary>
        /// 执行数量
        /// </summary>
        public static int ImplementNum = 0;
        /// <summary>
        /// 成功数量
        /// </summary>
        public static int SuccessNum = 0;
        /// <summary>
        /// 失败数量
        /// </summary>
        public static int ErrorNum = 0;
        /// <summary>
        /// 等待数量
        /// </summary>
        public static int WaitNum = 0;
        /// <summary>
        /// 取消操作通知
        /// </summary>
        private CancellationTokenSource _cancellationTokenSource;
        /// <summary>
        /// 程序运行状态
        /// </summary>
        public Boolean IsLoading = false;
        /// <summary>
        /// 程序执行任务的开始时间
        /// </summary>
        public long StartTime = 0;
        /// <summary>
        /// 程序执行任务的完成时间
        /// </summary>
        public long EndTime = 0;

        public Main()
        {
            InitializeComponent();
            this.init();
        }

        public void init()
        {
            UILogUtils.Add(this);
            dgv_user.AutoGenerateColumns = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        #region 右键导入账号
        /// <summary>
        /// 右键导入账号
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void 导入账号ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                var dataObject = Clipboard.GetDataObject();
                if (dataObject.GetDataPresent(DataFormats.Text))
                {
                    string text = (String)dataObject.GetData(DataFormats.Text);

                    var iqiyiAccounts = Text2IqiyiAccounts(text);

                    AddIqiyiAccount(iqiyiAccounts);
                }
                else
                {
                    UILogUtils.Error("粘贴板没有文本数据。");
                }
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }
        private List<IqiyiAccount> Text2IqiyiAccounts(string text)
        {
            var iqiyiAccounts = new List<IqiyiAccount>();
            var lines = text.Split(new String[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries).ToList();
            lines.ForEach(line =>
            {
                var optionItems = line.Split(new String[] { "----" }, StringSplitOptions.RemoveEmptyEntries);
                if (optionItems == null)
                {
                    throw new Exception("行:" + line + "格式不正确[1]。");
                }

                string version = optionItems[0];
                if (optionItems.Length != 3)
                {
                    throw new Exception("行:" + line + "格式不正确[2]。");
                }
                var iqiyiAccount = new IqiyiAccount()
                {
                    MobileNo = optionItems[0],
                    Password = optionItems[1],
                    Cookie = optionItems[2]
                };
                iqiyiAccounts.Add(iqiyiAccount);

            });
            return iqiyiAccounts;
        }
        /// <summary>
        /// 添加爱奇艺账号
        /// </summary>
        private void AddIqiyiAccount(List<IqiyiAccount> iqiyiAccounts)
        {

            if (iqiyiAccounts == null || iqiyiAccounts.Count < 0)
            {
                return;
            }
            IqiyiAccounts.AddRange(iqiyiAccounts);
            RefreshIqiyiAccount();
        }
        private void RefreshIqiyiAccount()
        {
            dgv_user.DataSource = null;
            dgv_user.DataSource = IqiyiAccounts;
        }
        #endregion
        #region 设置界面数量
        public void setStatusStripNum()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(setStatusStripNumNext));
            }
            else
            {
                setStatusStripNumNext();
            }
        }

        private void setStatusStripNumNext()
        {
            tssl_select_num.Text = SelectNum.ToString();
            tssl_implement_num.Text = ImplementNum.ToString();
            tssl_success_num.Text = SuccessNum.ToString();
            tssl_error_num.Text = ErrorNum.ToString();
            tssl_wait_num.Text = WaitNum.ToString();

            if (SelectNum == ImplementNum)
            {
                IsLoading = false;
                EndTime = DateUtils.GetTimeStampOfMilliseconds();
                var hour = Convert.ToInt32((EndTime - StartTime) / (1000 * 60 * 60));
                var mominute = Convert.ToInt32(((EndTime - StartTime) % (1000 * 60 * 60)) / (1000 * 60));
                var msecond = ((EndTime - StartTime) % (1000 * 60) / 1000);
                UILogUtils.Info($"本次共计执行{SelectNum}个账号,所用时长{hour}小时{mominute}分钟{msecond}秒钟");
            }
        }
        #endregion
        #region 界面日志

        public void Info(string message)
        {
            message = string.Format("[信息] [{0}] {1} {2}", DateTime.Now.ToString("HH:mm:ss"), message, Environment.NewLine);

            if (txtLog.InvokeRequired)
            {
                txtLog.Invoke(new Action<String>(ShowInfoMessage), new object[] { message });
            }
            else
            {
                ShowInfoMessage(message);
            }
        }

        private void ShowInfoMessage(string message)
        {
            txtLog.AppendText(message);
            // 将光标位置设置到当前内容的末尾
            txtLog.SelectionStart = txtLog.Text.Length;
            // 滚动到光标位置
            txtLog.ScrollToCaret();
        }

        public void Error(string message)
        {
            message = string.Format("[错误] [{0}] {1} {2}", DateTime.Now.ToString("HH:mm:ss"), message, Environment.NewLine);

            if (txtLog.InvokeRequired)
            {
                txtLog.Invoke(new Action<String>(ShowErrorMessage), new object[] { message });
            }
            else
            {
                ShowErrorMessage(message);
            }
        }

        public void Error(Exception exception)
        {
            String message = string.Format("[信息] [{0}] {1} {2}", DateTime.Now.ToString("HH:mm:ss"),
                isDebuger ? exception.ToString() : exception.Message, Environment.NewLine);

            if (txtLog.InvokeRequired)
            {
                txtLog.Invoke(new Action<String>(ShowErrorMessage), new object[] { message });
            }
            else
            {
                ShowErrorMessage(message);
            }
        }

        public void Error(string message, Exception exception)
        {
            Error(message);
            Error(exception);
        }

        private void ShowErrorMessage(string message)
        {
            txtLog.SelectionStart = txtLog.TextLength;
            txtLog.SelectionLength = 0;
            txtLog.SelectionColor = Color.Red;
            txtLog.AppendText(message);
            txtLog.SelectionColor = txtLog.ForeColor;
            //将光标位置设置到当前内容的末尾
            txtLog.SelectionStart = txtLog.Text.Length;
            //滚动到光标位置
            txtLog.ScrollToCaret();
        }

        public void Debug(string message)
        {
            if (!isDebuger)
            {
                return;
            }

            message = string.Format("[调试] [{0}] {1} {2}", DateTime.Now.ToString("HH:mm:ss"), message, Environment.NewLine);

            if (txtLog.InvokeRequired)
            {
                txtLog.Invoke(new Action<String>(ShowDebugMessage), new object[] { message });
            }
            else
            {
                ShowDebugMessage(message);
            }
        }
        private void ShowDebugMessage(string message)
        {
            txtLog.SelectionStart = txtLog.TextLength;
            txtLog.SelectionLength = 0;
            txtLog.SelectionColor = Color.Blue;
            txtLog.AppendText(message);
            txtLog.SelectionColor = txtLog.ForeColor;
            //将光标位置设置到当前内容的末尾
            txtLog.SelectionStart = txtLog.Text.Length;
            //滚动到光标位置
            txtLog.ScrollToCaret();
        }
        #endregion
        #region 签到
        private void 签到ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv_user.SelectedRows == null || dgv_user.SelectedRows.Count <= 0)
                {
                    UILogUtils.Error("请选择要签到的账号。");
                    return;
                }

                var iqiyiAccounts = new List<IqiyiAccount>();

                foreach (DataGridViewRow row in dgv_user.SelectedRows)
                {
                    IqiyiAccount iqiyiAccount = row.DataBoundItem as IqiyiAccount;
                    iqiyiAccounts.Add(iqiyiAccount);
                }

                AccountSignIn(iqiyiAccounts);

            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }
        public void AccountSignIn(List<IqiyiAccount> iqiyiAccounts)
        {
            if (IsLoading)
            {
                return;
            }
            IsLoading = true;

            StartTime = DateUtils.GetTimeStampOfMilliseconds();

            SelectNum = iqiyiAccounts.Count;
            ImplementNum = 0;
            SuccessNum = 0;
            ErrorNum = 0;
            WaitNum = iqiyiAccounts.Count;
            setStatusStripNum();

            _cancellationTokenSource?.Cancel();
            _cancellationTokenSource = new CancellationTokenSource();
            CancellationToken cancellationToken = _cancellationTokenSource.Token;

            String fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志","签到", DateTime.Now.ToString("yyyyMMdd"));
            String fileNamePrefixSuccess = "签到成功-" + DateTime.Now.ToString("HHmmss") + ".txt";
            String fileNamePrefixError = "签到失败-" + DateTime.Now.ToString("HHmmss") + ".txt";

            foreach (var iqiyiAccount in iqiyiAccounts)
            {
                Task.Factory.StartNew(() =>
                {
                    string filePath = "";
                    string fileText = $"{iqiyiAccount.MobileNo}----{iqiyiAccount.Password}----{iqiyiAccount.Cookie}";
                    if (User.UserDoSign(iqiyiAccount))
                    {
                        iqiyiAccount.SignStatus = 1;
                        iqiyiAccount.SignStatusDesc = "签到成功";

                        Interlocked.Increment(ref SuccessNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixSuccess);
                    }
                    else
                    {
                        iqiyiAccount.SignStatus = 2;
                        iqiyiAccount.SignStatusDesc = "签到错误";
                        Interlocked.Increment(ref ErrorNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixError);
                    }
                    setStatusStripNum();

                    FileUtils.AppendAllText(filePath, fileText , Encoding.UTF8);

                },  cancellationToken);
            }
        }
        #endregion
        #region 查询观影豆
        private void 查询观影豆ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv_user.SelectedRows == null || dgv_user.SelectedRows.Count <= 0)
                {
                    UILogUtils.Error("请选择要查询的账号。");
                    return;
                }

                var iqiyiAccounts = new List<IqiyiAccount>();
                foreach (DataGridViewRow row in dgv_user.SelectedRows)
                {
                    IqiyiAccount iqiyiAccount = row.DataBoundItem as IqiyiAccount;
                    iqiyiAccounts.Add(iqiyiAccount);
                }

                queryMovieBean(iqiyiAccounts);
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }
        public void queryMovieBean(List<IqiyiAccount> iqiyiAccounts)
        {
            if (IsLoading)
            {
                return;
            }
            IsLoading = true;

            StartTime = DateUtils.GetTimeStampOfMilliseconds();

            SelectNum = iqiyiAccounts.Count;
            ImplementNum = 0;
            SuccessNum = 0;
            ErrorNum = 0;
            WaitNum = iqiyiAccounts.Count;
            setStatusStripNum();

            _cancellationTokenSource?.Cancel();
            _cancellationTokenSource = new CancellationTokenSource();
            CancellationToken cancellationToken = _cancellationTokenSource.Token;

            String fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志","查询观影豆", DateTime.Now.ToString("yyyyMMdd"));
            String fileNamePrefixSuccess = "查询观影豆成功-" + DateTime.Now.ToString("HHmmss") + ".txt";
            String fileNamePrefixError = "查询观影豆失败-" + DateTime.Now.ToString("HHmmss") + ".txt";

            foreach (var iqiyiAccount in iqiyiAccounts)
            {
                Task.Factory.StartNew(() =>
                {
                    string filePath = "";
                    string fileText = $"{iqiyiAccount.MobileNo}----{iqiyiAccount.Password}----{iqiyiAccount.Cookie}";

                    var iqiyiAccountMovieBean = User.GetMovieBean(iqiyiAccount);
                    iqiyiAccount.MovieBean = Convert.ToInt32(iqiyiAccountMovieBean);

                    if (iqiyiAccountMovieBean == "-1")
                    {
                        Interlocked.Increment(ref ErrorNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixError);
                    }
                    else
                    {
                        Interlocked.Increment(ref SuccessNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixSuccess);
                    }

                    setStatusStripNum();

                    FileUtils.AppendAllText(filePath, fileText, Encoding.UTF8);

                }, cancellationToken);
            }
        }
        #endregion

        private void 查询优惠券ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv_user.SelectedRows == null || dgv_user.SelectedRows.Count <= 0)
                {
                    UILogUtils.Error("请选择要查询的账号。");
                    return;
                }

                var iqiyiAccounts = new List<IqiyiAccount>();
                foreach (DataGridViewRow row in dgv_user.SelectedRows)
                {
                    IqiyiAccount iqiyiAccount = row.DataBoundItem as IqiyiAccount;
                    iqiyiAccounts.Add(iqiyiAccount);
                }
                queryCouponNum(iqiyiAccounts);
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }

        public void queryCouponNum(List<IqiyiAccount> iqiyiAccounts)
        {
            if (IsLoading)
            {
                return;
            }
            IsLoading = true;

            StartTime = DateUtils.GetTimeStampOfMilliseconds();

            SelectNum = iqiyiAccounts.Count;
            ImplementNum = 0;
            SuccessNum = 0;
            ErrorNum = 0;
            WaitNum = iqiyiAccounts.Count;
            setStatusStripNum();

            _cancellationTokenSource?.Cancel();
            _cancellationTokenSource = new CancellationTokenSource();
            CancellationToken cancellationToken = _cancellationTokenSource.Token;

            String fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志","查询优惠券", DateTime.Now.ToString("yyyyMMdd"));
            String fileNamePrefixSuccess = "查询优惠券成功-" + DateTime.Now.ToString("HHmmss") + ".txt";
            String fileNamePrefixError = "查询优惠券失败-" + DateTime.Now.ToString("HHmmss") + ".txt";

            foreach (var iqiyiAccount in iqiyiAccounts)
            {
                Task.Factory.StartNew(() =>
                {
                    string filePath = "";
                    string fileText = $"{iqiyiAccount.MobileNo}----{iqiyiAccount.Password}----{iqiyiAccount.Cookie}";

                    var iqiyiAccountCouponNum = User.GetCouponList(iqiyiAccount);
                    iqiyiAccount.CouponNum = Convert.ToInt32(iqiyiAccountCouponNum);

                    if (iqiyiAccountCouponNum == "-1")
                    {
                        Interlocked.Increment(ref ErrorNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixError);
                    }
                    else
                    {
                        Interlocked.Increment(ref SuccessNum);
                        Interlocked.Increment(ref ImplementNum);
                        Interlocked.Decrement(ref WaitNum);

                        filePath = Path.Combine(fileDirectory, fileNamePrefixSuccess);
                    }

                    setStatusStripNum();

                    FileUtils.AppendAllText(filePath, fileText, Encoding.UTF8);
                });
            }
        }

        private void Btn_start_Click(object sender, EventArgs e)
        {
            if (dgv_user.Rows.Count == 0)
            {
                UILogUtils.Info("请先导入账号");
                return;
            }

            // 执行签到
            if (rb_signIn.Checked)
            {
                AccountSignIn(IqiyiAccounts);
            }
            // 执行查询观影豆
            else if (rb_movieBean.Checked)
            {
                queryMovieBean(IqiyiAccounts);
            }
            // 执行查询优惠券
            else if (rb_coupon.Checked)
            {
                queryCouponNum(IqiyiAccounts);
            }
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            try
            {
                _cancellationTokenSource?.Cancel();
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex.ToString());
            }
            IsLoading = false;
        }

        private void 复制账号ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv_user.SelectedRows == null || dgv_user.SelectedRows.Count <= 0)
                {
                    UILogUtils.Error("请选择要查询的账号。");
                    return;
                }

                var iqiyiAccounts = new List<IqiyiAccount>();

                foreach (DataGridViewRow row in dgv_user.SelectedRows)
                {
                    IqiyiAccount iqiyiAccount = row.DataBoundItem as IqiyiAccount;
                    iqiyiAccounts.Add(iqiyiAccount);
                }

                copyAccountInfo(iqiyiAccounts);
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }
        private void copyAccountInfo(List<IqiyiAccount> iqiyiAccounts)
        {

            StringBuilder stringBuilder = new StringBuilder();

            foreach (var iqiyiAccount in iqiyiAccounts)
            {
                stringBuilder.AppendLine($"{iqiyiAccount.MobileNo}----{iqiyiAccount.Password}----{iqiyiAccount.Cookie}");
            }

            Clipboard.SetText(stringBuilder.ToString());

            UILogUtils.Info("复制账号成功。");
        }

        private void 导出账号ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(dgv_user.Rows.Count == 0)
            {
                UILogUtils.Info($"请先导入账号");
                return;
            }

            var iqiyiAccounts = new List<IqiyiAccount>();
            foreach(DataGridViewRow row in dgv_user.Rows)
            {
                IqiyiAccount iqiyiAccount = row.DataBoundItem as IqiyiAccount;
                iqiyiAccounts.Add(iqiyiAccount);
            }

            copyAccountInfo(iqiyiAccounts);
        }

        private void 清空账号ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            dgv_user.DataSource = null;
        }

        private void 删除ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv_user.SelectedRows == null || dgv_user.SelectedRows.Count <= 0)
                {
                    UILogUtils.Error("请选择要删除的账号。");
                    return;
                }

                IList list = (IList)dgv_user.DataSource;
                foreach (DataGridViewRow row in dgv_user.SelectedRows)
                {
                    list.RemoveAt(row.Index);//移除
                }
                dgv_user.DataSource = null; //为空
                dgv_user.DataSource = list; //显示数据

                UILogUtils.Info("删除账号成功。");
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex);
            }
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            try
            {
                String fileDirectory = "";
                if (rb_signIn.Checked)
                {
                    fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志", "签到", DateTime.Now.ToString("yyyyMMdd"));
                }
                else if (rb_movieBean.Checked)
                {
                    fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志", "查询观影豆", DateTime.Now.ToString("yyyyMMdd"));
                }
                else if (rb_coupon.Checked)
                {
                    fileDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "日志","查询优惠券", DateTime.Now.ToString("yyyyMMdd"));
                }

                if (!Directory.Exists(fileDirectory))
                {
                    UILogUtils.Error("文件夹不存在。" + fileDirectory);
                    return;
                }
                System.Diagnostics.Process.Start("explorer.exe", fileDirectory);
            }
            catch (Exception ex)
            {
                UILogUtils.Error(ex.ToString());
            }
        }
    }
}
